""" surl.services """

#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

import xml.dom.minidom
import json

from surl import surl

def supportedServices():
    """ Returns a dictionary with the supported surl services """
    dict_service = {
        'a.gd'         : a_gd(),
        'bit.ly'       : bit_ly(),
        'burnurl.com'  : burnurl_com(),
        'cli.gs'       : cli_gs(),
        'decenturl.com': decenturl_com(),
        'digg.com'     : digg_com(),
        'is.gd'        : is_gd(),
        'kl.am'        : kl_am(),
        'sn.im'        : sn_im('sn.im'),
        'snipr.com'    : sn_im('snipr.com'),
        'snipurl.com'  : sn_im('snipurl.com'),
        'snurl.com'    : sn_im('snurl.com'),
        'tinyurl.com'  : tinyurl_com(),
        'tr.im'        : tr_im(),
    }
    return dict_service

class tinyurl_com:
    #http://tinyurl.com/api-create.php?url=http://example.com/
    def get(self, args, url):
        values = {
            'url': url,
        }
        service_link = "http://tinyurl.com/api-create.php"
        result = surl().getShortenedLink(service_link, url, values)
        return result


class tr_im:
    #http://api.tr.im/api/trim_simple?url=http://www.example.com
    def get(self, args, url):
        values = {
            'url': url,
        }
        service_link = "http://api.tr.im/api/trim_simple"
        result = surl().getShortenedLink(service_link, url, values)
        return result

class burnurl_com:
    #http://burnurl.com/?url=google.com&output=plain
    def get(self, args, url):
        values = {
            'url'   : url,
            'output': 'plain',
        }
        service_link = "http://burnurl.com/"
        result = surl().getShortenedLink(service_link, url, values)
        return result

class sn_im:
    #http://snipr.com/site/getsnip?sniplink=http://www.example.com&snipformat=simple&snipuser=someuser&snipapi=someapi
    def __init__(self, service_domain):
        self.service_link = 'http://' + service_domain + '/site/getsnip'

    def get(self, args, url):
        values = {
            'sniplink'  : url,
            'snipformat': 'simple',
            'snipapi'   : args['api-key'],
            'snipuser'  : args['username'],
        }
        result = surl().getShortenedLink(self.service_link, url, values)
        return result

class is_gd:
    #http://is.gd/api.php?longurl=http://www.example.com
    #GET
    def get(self, args, url):
        service_link = "http://is.gd/api.php"
        # hack over it, don't use POST
        result = surl().getShortenedLink(service_link + "?longurl=%s" % url, url)
        return result

class a_gd:
    #curl "http://a.gd/?module=ShortURL&file=Add&mode=API" --data "url=http://example.com"
    #POST
    def get(self, args, url):
        values = {
            'url': url,
        }
        service_link = "http://a.gd/?module=ShortURL&file=Add&mode=API"
        result = surl().getShortenedLink(service_link, url, values)
        #<h3>URL has wrong format! Forgot 'http://'?</h3>
        if re.match('<h3>', result):
            result = url
        return result

class digg_com:
    #curl "http://services.digg.com/url/short/create?url=http%3A%2F%2Fwww.google.com&appkey=http://launchpad.net/surl"
    #GET
    def parseXML(self, xml_string):
        doc = xml.dom.minidom.parseString(xml_string)
        first_shorturl = doc.getElementsByTagName("shorturl")[0]
        return first_shorturl.getAttribute("short_url")

    def get(self, args, url):
        appkey = args['api-key']
        if not appkey:
            appkey = 'http://launchpad.net/surl'

        service_link = "http://services.digg.com/url/short/create?appkey=%s&url=%s"
        url_response = surl().getShortenedLink(service_link % (appkey, url), url)
        if not url_response == url:
            #i.e. If there was no error
            result = self.parseXML(url_response)
        else:
            result = url
        return result

class decenturl_com:
    #curl "http://decenturl.com/api-get?u=youtube.com%2Fwatch%3Fv%3DpQHX-SjgQvQ&t=medieval"
    #GET
    def parseJSON(self, json_string):
        array = json.loads(json_string)
        #print(array)
        if array[0] == "ok":
            result = array[3]
        else:
            result = self.url
        return result

    def get(self, args, url):
        self.url = url
        service_link = "http://decenturl.com/api-get?u=%s"
        url_response = surl().getShortenedLink(service_link % url, url)
        if not url_response == url:
            #i.e. If there was no error
            #["ok", "example/example-web-page-2", "http://example.com/", "http://example.decenturl.com/example-web-page-2"]
            result = self.parseJSON(url_response)
        else:
            result = url
        return result

class kl_am:
    #curl "http://kl.am/api/shorten/?format=text&url=http%3A%2F%2Fgoogle.com"
    #GET
    def get(self, args, url):
        service_link = "http://kl.am/api/shorten/?format=text&url=%s"
        result = surl().getShortenedLink(service_link % url, url)
        return result

class cli_gs:
    #curl "http://cli.gs/api/v1/cligs/create?url=http://www.example.com&appid=http://launchpad.net/surl"
    #GET
    def get(self, args, url):
        appid = args['api-key']
        if not appid:
            appid = 'http://launchpad.net/surl'

        service_link = "http://cli.gs/api/v1/cligs/create?appid=%s&url=%s"
        result = surl().getShortenedLink(service_link % (appid, url), url)
        return result

class bit_ly:
    #curl "http://api.bit.ly/shorten?version=2.0.1&longUrl=http://www.google.com&format=json&login=bitlyapidemo&apiKey=R_0da49e0a9118ff35f52f629d2d71bf07"
    #GET
    def parseJSON(self, json_string):
        array = json.loads(json_string)
        #print(array)
        try:
            shorturl = array['results'][self.url]['shortUrl']
        except KeyError:
            #no shortUrl: errorCode > 0 or new API?
            return self.url

        if array['statusCode'] == "OK" and array['errorCode'] == 0:
            result = shorturl
        else:
            result = self.url

        return result

    def get(self, args, url):
        self.url = url
        api_version = '2.0.1'
        username = args['username']
        apikey = args['api-key']
        if not username or not apikey:
            if not args['quiet']:
                print("ERROR: bit.ly requires a username and an API key")
            return url

        service_link = "http://api.bit.ly/shorten?version=%s&login=%s&apiKey=%s&longUrl=%s&format=json"
        url_response = surl().getShortenedLink(service_link % (api_version, username, apikey, url), url)
        if not url_response == url:
            #i.e. If there was no error
            """ Example:
            {
                "errorCode": 0, 
                "errorMessage": "", 
                "results": {
                    "http://www.google.com": { "hash": "2V6CFi", "shortKeywordUrl": "", "shortUrl": "http://bit.ly/8ttn", "userHash": "8ttn" }
                }, 
                "statusCode": "OK"
            }
            """
            result = self.parseJSON(url_response)
        else:
            result = url
        return result
